#! python
# -*- coding: utf-8 -*-

"""Package for sequence related classes and objects.

Authors:
    * Daniel Cosmo Pizetta <daniel.pizetta@usp.br>

Authors:
    2015/11/01

Todo:
    Fix these modules.

"""
